using System.Security.Cryptography;
using System.Text;

namespace Spyun
{
    public class Api
    {
        public static string baseUri = "https://open.spyun.net/v1/";

        private string appid;

        private string appsecret;

        private HttpClient httpClient;

        public Api(string appid, string appsecret) 
        {
            this.appid = appid;
            this.appsecret = appsecret;
            httpClient = new HttpClient();
            httpClient.Timeout = TimeSpan.FromSeconds(5);
        }

        // 添加打印机
        public string addPrinter(string sn, string pkey, string name)
        {
            var param = new Dictionary<string, string>();
            param.Add("sn", sn);
            param.Add("pkey", pkey);
            param.Add("name", name);

            return request("POST", "printer/add", param).Result;
        }

        // 删除打印机
        public string deletePrinter(string sn)
        {
            var param = new Dictionary<string, string>();
            param.Add("sn", sn);

            return request("DELETE", "printer/delete", param).Result;
        }

        // 修改打印机信息
        public string updatePrinter(string sn, string name)
        {
            var param = new Dictionary<string, string>();
            param.Add("sn", sn);
            param.Add("name", name);

            return request("PATCH", "printer/update", param).Result;
        }

        // 修改打印机参数
        public string updatePrinterSetting(string sn, int auto_cut, string voice)
        {
            var param = new Dictionary<string, string>();
            param.Add("sn", sn);
            param.Add("auto_cut", auto_cut.ToString());
            param.Add("voice", voice);

            return request("PATCH", "printer/setting", param).Result;
        }

        // 获取打印机信息
        public string getPrinter(string sn)
        {
            var param = new Dictionary<string, string>();
            param.Add("sn", sn);

            return request("GET", "printer/info", param).Result;
        }

        // 打印订单
        public string print(string sn, string content, int times)
        {
            var param = new Dictionary<string, string>();
            param.Add("sn", sn);
            param.Add("content", content);
            param.Add("times", times.ToString());

            return request("POST", "printer/print", param).Result;
        }

        // 清空待打印订单
        public string deletePrints(string sn)
        {
            var param = new Dictionary<string, string>();
            param.Add("sn", sn);

            return request("DELETE", "printer/cleansqs", param).Result;
        }

        // 查询打印订单状态
        public string getPrintsStatus(string id)
        {
            var param = new Dictionary<string, string>();
            param.Add("id", id);

            return request("GET", "printer/order/status", param).Result;
        }

        // 查询打印机历史打印订单数
        public string getPrintsOrders(string sn, string date)
        {
            var param = new Dictionary<string, string>();
            param.Add("sn", sn);
            param.Add("date", date);

            return request("GET", "printer/order/number", param).Result;
        }

        // 发送请求
        private async Task<string> request(string method, string uri, Dictionary<string, string> param)
        {
            // 公共请求参数
            param.Add("appid", appid);
            param.Add("timestamp", appid);
            param.Add("sign", makeSign(param));
            var url = baseUri + uri;
            var content = new FormUrlEncodedContent(param);
            
            HttpResponseMessage res;
            if (method.Equals("GET")) {
                res = await httpClient.GetAsync(url + "?" + content.ReadAsStringAsync().Result);
            } else if (method.Equals("DELETE")) {
                res =  await httpClient.DeleteAsync(url + "?" + content.ReadAsStringAsync().Result);
            } else if (method.Equals("POST")) {
                res =  await httpClient.PostAsync(url, content);
            } else if (method.Equals("PUT")) {
                res =  await httpClient.PutAsync(url, content);
            } else if (method.Equals("PATCH")) {
                res =  await httpClient.PatchAsync(url, content);
            } else {
                throw new ApplicationException("method not supported");
            }
            
            if (res.StatusCode != System.Net.HttpStatusCode.OK)
            {
                throw new ApplicationException(res.Content.ReadAsStringAsync().Result);
            }
            
            return res.Content.ReadAsStringAsync().Result;
        }

        // 创建签名
        public string makeSign(Dictionary<string, string> param)
        {
            string[] keys = param.Keys.ToArray();
            Array.Sort(keys);
            
            string sign = "";
            for (int i = 0; i < keys.Length; i++)
            {
                string v = param[keys[i]];
                if (!keys[i].Equals("sign") && !keys[i].Equals("appsecret") && v != null && !v.Equals(""))
                {
                    if (i > 0) {
                        sign += "&";
                    }
                    sign += keys[i] + "=" + v;
                }
            }
            sign += "&appsecret=" + appsecret;

            var md5Hasher = MD5.Create();
            var data = md5Hasher.ComputeHash(Encoding.UTF8.GetBytes(sign));
            var sBuilder = new StringBuilder();
            for (int i = 0; i < data.Length; i++)
            {
                sBuilder.Append(data[i].ToString("x2"));
            }
            
            return sBuilder.ToString().ToUpper();
        }
    }
}